#ifndef MATH_UTILS_H
#define MATH_UTILS_H

/* needed for fabs, sqrt() below */
#include <cmath>
namespace TNT{

	/**
		@returns hypotenuse of real (non-complex) scalars a and b by 
		avoiding underflow/overflow
		using (a * sqrt( 1 + (b/a) * (b/a))), rather than
		sqrt(a*a + b*b).
	*/
	// Modified by JTB to check which is larger (a or b) and use different calculations based on that.
	template <class Real>
	Real hypot(const Real &a, const Real &b){
		if( a==0 )
			return fabs(b);
		if( b==0 ) 
			return fabs(a);
		Real absa=fabs(a);
		Real absb=fabs(b);
		if( absa>absb ) {
			Real c = b/a;
			return absa * sqrt(1 + c*c);
			}
		Real c = a/b;
		return absb * sqrt(1 + c*c);
		}
		
	} /* TNT namespace */

#endif	/* MATH_UTILS_H */
